/***********************************************************
 * $Id$
 *
 * FancyDB
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDBSTATEMENT_H_
#define FANCYDBSTATEMENT_H_

#include <fancydbconnectionbound.h>

namespace Fancy {

namespace DB {

class Result;

/**
 * An ordinary SQL statement.
 */
class FANCYDB_API Statement: public virtual ConnectionBound {
public:
	typedef Fancy::ObjPtr<Statement> ptr_t;

	/**
	 * Set the SQL statement to be executed. If this is a
	 * @c PreparedStatement, the SQL statement may contain
	 * question marks as placeholders.
	 */
	virtual void setSQL(const std::string &s) = 0;

	/**
	 * Execute the statement as an update or insert, which require
	 * no result set.
	 */
	virtual void executeUpdate() = 0;

	/**
	 * Execute the statement as a query, which returns a result set.
	 * @return The result set or @c null, if the SQL statement was not
	 *         a query.
	 */
	virtual Fancy::ObjPtr<Result> executeQuery() = 0;

	/**
	 * Register an automatically generated INT/BIGINT column whose generated value shall be retrieved using @getGeneratedIdValue() later.
	 * Most drivers will not support more than one such column an throw an exception if a 2nd such column is registered.
	 */
	virtual void addGeneratedIdColumn(const std::string &id) = 0;

	/**
	 * Returns the automatically generated value of a column previously registered using @addGeneratedIdColumn().
	 * @return The value of the column just generated by an executeUpdate("INSERT ...") call. 0 in case of any problems.
	 */
	virtual uint64_t getGeneratedIdValue(const std::string &id) = 0;

protected:
	Statement();
	virtual ~Statement();
};

}

}

#endif /* FANCYDBSTATEMENT_H_ */
